import unittest
from datetime import date
from unittest.mock import Mock

from promissory_note.commands import IssuePromissoryNoteCommand
from promissory_note.use_cases import IssuePromissoryNote


class IssuePromissoryNoteTests(unittest.TestCase):
    def setUp(self):
        self._image_generation_service = Mock()
        self._email_service = Mock()

        self._issue_promissory_note_command = IssuePromissoryNoteCommand(number=1,
                                                                         due_date=date.today(),
                                                                         value=100.99,
                                                                         beneficiary_name='Steve',
                                                                         beneficiary_cpf='11111111199',
                                                                         beneficiary_email='captain@marvel.com',
                                                                         currency='real',
                                                                         city_payment='New York',
                                                                         state_payment='New York',
                                                                         emitter_name='Tony Stark',
                                                                         emitter_cpf='11111111188',
                                                                         emitter_address='New York',
                                                                         emitter_email='iron_man@marvel.com',
                                                                         issuance_date=date.today())

        self._issue_promissory_note = IssuePromissoryNote(
            image_generation_service=self._image_generation_service,
            email_service=self._email_service)

    def test_should_image_generation_service_is_called_when_promissory_note_is_issued(self):
        self._issue_promissory_note.execute(command=self._issue_promissory_note_command)

        self._image_generation_service.assert_called_once()

    def test_should_email_service_is_called_when_promissory_note_is_issued(self):
        self._issue_promissory_note.execute(command=self._issue_promissory_note_command)

        self._email_service.assert_called_once()



import unittest
from datetime import date

from promissory_note.commands import IssuePromissoryNoteCommand


class IssuePromissoryNoteCommandTests(unittest.TestCase):
    def setUp(self):
        self._issue_promissory_note_command = IssuePromissoryNoteCommand(number=1,
                                                                         due_date=date.today(),
                                                                         value=100.99,
                                                                         beneficiary_name='Steve',
                                                                         beneficiary_cpf='11111111199',
                                                                         beneficiary_email='captain@marvel.com',
                                                                         currency='real',
                                                                         city_payment='New York',
                                                                         state_payment='New York',
                                                                         emitter_name='Tony Stark',
                                                                         emitter_cpf='11111111188',
                                                                         emitter_address='New York',
                                                                         emitter_email='iron_man@marvel.com',
                                                                         issuance_date=date.today())

    def test_should_issue_promissory_note_command_contains_number(self):
        self.assertEqual(1, self._issue_promissory_note_command.number)

    def test_should_issue_promissory_note_command_contains_due_date(self):
        self.assertEqual(date.today(), self._issue_promissory_note_command.due_date)

    def test_should_issue_promissory_note_command_contains_value(self):
        self.assertEqual(100.99, self._issue_promissory_note_command.value)

    def test_should_issue_promissory_note_command_contains_beneficiary_name(self):
        self.assertEqual('Steve', self._issue_promissory_note_command.beneficiary_name)

    def test_should_issue_promissory_note_command_contains_beneficiary_cpf(self):
        self.assertEqual('11111111199', self._issue_promissory_note_command.beneficiary_cpf)

    def test_should_issue_promissory_note_command_contains_beneficiary_email(self):
        self.assertEqual('captain@marvel.com', self._issue_promissory_note_command.beneficiary_email)

    def test_should_issue_promissory_note_command_contains_currency(self):
        self.assertEqual('real', self._issue_promissory_note_command.currency)

    def test_should_issue_promissory_note_command_contains_city_payment(self):
        self.assertEqual('New York', self._issue_promissory_note_command.city_payment)

    def test_should_issue_promissory_note_command_contains_state_payment(self):
        self.assertEqual('New York', self._issue_promissory_note_command.state_payment)

    def test_should_issue_promissory_note_command_contains_emitter_name(self):
        self.assertEqual('Tony Stark', self._issue_promissory_note_command.emitter_name)

    def test_should_issue_promissory_note_command_contains_emitter_cpf(self):
        self.assertEqual('11111111188', self._issue_promissory_note_command.emitter_cpf)

    def test_should_issue_promissory_note_command_contains_emitter_address(self):
        self.assertEqual('New York', self._issue_promissory_note_command.emitter_address)

    def test_should_issue_promissory_note_command_contains_emitter_email(self):
        self.assertEqual('iron_man@marvel.com', self._issue_promissory_note_command.emitter_email)

    def test_should_issue_promissory_note_command_contains_issuance_date(self):
        self.assertEqual(date.today(), self._issue_promissory_note_command.issuance_date)



import unittest
from datetime import date, timedelta
from unittest.mock import Mock

from promissory_note.entities import Beneficiary, Emitter, PromissoryNote
from promissory_note.events import PromissoryNoteIssued, PromissoryNoteNotIssued
from promissory_note.value_objects import Name, Email, Cpf


class BeneficiaryTests(unittest.TestCase):
    def setUp(self):
        name = Name('Steve')
        cpf = Cpf('11111111199')
        email = Email('captain@marvel.com')

        self._beneficiary = Beneficiary(name=name,
                                        cpf=cpf,
                                        email=email)

    def test_should_is_valid_true_when_all_properties_are_valid(self):
        self.assertTrue(self._beneficiary.is_valid)

    def test_should_is_valid_false_when_name_is_invalid(self):
        name = Name(None)
        cpf = Cpf('11111111199')
        email = Email('captain@marvel.com')

        beneficiary = Beneficiary(name=name,
                                  cpf=cpf,
                                  email=email)

        self.assertFalse(beneficiary.is_valid)

    def test_should_is_valid_false_when_cpf_is_invalid(self):
        name = Name('Steve')
        cpf = Cpf('1111111119')
        email = Email('captain@marvel.com')

        beneficiary = Beneficiary(name=name,
                                  cpf=cpf,
                                  email=email)

        self.assertFalse(beneficiary.is_valid)

    def test_should_is_valid_false_when_email_is_invalid(self):
        name = Name('Steve')
        cpf = Cpf('11111111199')
        email = Email('invalid email')

        beneficiary = Beneficiary(name=name,
                                  cpf=cpf,
                                  email=email)

        self.assertFalse(beneficiary.is_valid)

    def test_should_contains_property_name(self):
        self.assertEqual('Steve', self._beneficiary.name)

    def test_should_contains_property_cpf(self):
        self.assertEqual('11111111199', self._beneficiary.cpf)

    def test_should_contains_property_email(self):
        self.assertEqual('captain@marvel.com', self._beneficiary.email)


class EmitterTests(unittest.TestCase):
    def setUp(self):
        name = Name('Steve')
        cpf = Cpf('11111111199')
        address = 'New York'
        email = Email('captain@marvel.com')

        self._emitter = Emitter(name=name,
                                cpf=cpf,
                                address=address,
                                email=email)

    def test_should_is_valid_true_when_all_properties_are_valid(self):
        self.assertTrue(self._emitter.is_valid)

    def test_should_is_valid_false_when_name_is_invalid(self):
        name = Name('')
        cpf = Cpf('11111111199')
        address = 'New York'
        email = Email('captain@marvel.com.br')

        emitter = Emitter(name=name,
                          cpf=cpf,
                          address=address,
                          email=email)

        self.assertFalse(emitter.is_valid)

    def test_should_is_valid_false_when_cpf_is_invalid(self):
        name = Name('Steve')
        cpf = Cpf('1111111119')
        address = 'New York'
        email = Email('captain@marvel.com.br')

        emitter = Emitter(name=name,
                          cpf=cpf,
                          address=address,
                          email=email)

        self.assertFalse(emitter.is_valid)

    def test_should_is_valid_false_when_address_is_invalid(self):
        name = Name('Steve')
        cpf = Cpf('11111111199')
        address = ''
        email = Email('captain@marvel.com.br')

        emitter = Emitter(name=name,
                          cpf=cpf,
                          address=address,
                          email=email)

        self.assertFalse(emitter.is_valid)

    def test_should_notifications_must_contain_invalid_address_message_when_address_is_invalid(self):
        name = Name('Steve')
        cpf = Cpf('11111111199')
        address = ''
        email = Email('captain@marvel.com.br')

        emitter = Emitter(name=name,
                          cpf=cpf,
                          address=address,
                          email=email)

        invalid_address_notification = emitter.notifications[0]

        self.assertEqual('invalid address', invalid_address_notification.message)

    def test_should_is_valid_false_when_email_is_invalid(self):
        name = Name('Steve')
        cpf = Cpf('11111111199')
        address = 'New York'
        email = Email('invalid email')

        emitter = Emitter(name=name,
                          cpf=cpf,
                          address=address,
                          email=email)

        self.assertFalse(emitter.is_valid)

    def test_should_contains_property_name(self):
        self.assertEqual('Steve', self._emitter.name)

    def test_should_contains_property_cpf(self):
        self.assertEqual('11111111199', self._emitter.cpf)

    def test_should_contains_property_address(self):
        self.assertEqual('New York', self._emitter.address)

    def test_should_contains_property_email(self):
        self.assertEqual('captain@marvel.com', self._emitter.email)


class PromissoryNoteTests(unittest.TestCase):
    def setUp(self):
        number = 1
        due_date = date.today()
        value = 100.99
        currency = 'real'
        city_payment = 'New York'
        state_payment = 'New York'
        issuance_date = date.today()

        beneficiary = Beneficiary(name=Name('Steve'),
                                  cpf=Cpf('11111111199'),
                                  email=Email('captain@marvel.com'))

        emitter = Emitter(name=Name('Tony Stark'),
                          cpf=Cpf('11111111188'),
                          address='New York',
                          email=Email('iron_man@marvel.com'))

        self._promissory_note = PromissoryNote(number=number,
                                               due_date=due_date,
                                               value=value,
                                               currency=currency,
                                               city_payment=city_payment,
                                               state_payment=state_payment,
                                               issuance_date=issuance_date,
                                               beneficiary=beneficiary,
                                               emitter=emitter)

    def test_should_is_valid_true_when_all_properties_are_valid(self):
        self.assertTrue(self._promissory_note.is_valid)

    def test_should_is_valid_false_when_number_is_invalid(self):
        number = 0
        due_date = date.today()
        value = 100.99
        currency = 'real'
        city_payment = 'New York'
        state_payment = 'New York'
        issuance_date = date.today()

        beneficiary = Beneficiary(name=Name('Steve'),
                                  cpf=Cpf('11111111199'),
                                  email=Email('captain@marvel.com'))

        emitter = Emitter(name=Name('Tony Stark'),
                          cpf=Cpf('11111111188'),
                          address='New York',
                          email=Email('iron_man@marvel.com.br'))

        promissory_note = PromissoryNote(number=number,
                                         due_date=due_date,
                                         value=value,
                                         currency=currency,
                                         city_payment=city_payment,
                                         state_payment=state_payment,
                                         issuance_date=issuance_date,
                                         beneficiary=beneficiary,
                                         emitter=emitter)

        self.assertFalse(promissory_note.is_valid)

    def test_should_is_valid_false_when_due_date_is_invalid(self):
        number = 1
        due_date = date.today() - timedelta(days=1)
        value = 100.99
        currency = 'real'
        city_payment = 'New York'
        state_payment = 'New York'
        issuance_date = date.today()

        beneficiary = Beneficiary(name=Name('Steve'),
                                  cpf=Cpf('11111111199'),
                                  email=Email('captain@marvel.com'))

        emitter = Emitter(name=Name('Tony Stark'),
                          cpf=Cpf('11111111188'),
                          address='New York',
                          email=Email('iron_man@marvel.com.br'))

        promissory_note = PromissoryNote(number=number,
                                         due_date=due_date,
                                         value=value,
                                         currency=currency,
                                         city_payment=city_payment,
                                         state_payment=state_payment,
                                         issuance_date=issuance_date,
                                         beneficiary=beneficiary,
                                         emitter=emitter)

        self.assertFalse(promissory_note.is_valid)

    def test_should_is_valid_false_when_value_is_invalid(self):
        number = 1
        due_date = date.today()
        value = 0
        currency = 'real'
        city_payment = 'New York'
        state_payment = 'New York'
        issuance_date = date.today()

        beneficiary = Beneficiary(name=Name('Steve'),
                                  cpf=Cpf('11111111199'),
                                  email=Email('captain@marvel.com'))

        emitter = Emitter(name=Name('Tony Stark'),
                          cpf=Cpf('11111111188'),
                          address='New York',
                          email=Email('iron_man@marvel.com.br'))

        promissory_note = PromissoryNote(number=number,
                                         due_date=due_date,
                                         value=value,
                                         currency=currency,
                                         city_payment=city_payment,
                                         state_payment=state_payment,
                                         issuance_date=issuance_date,
                                         beneficiary=beneficiary,
                                         emitter=emitter)

        self.assertFalse(promissory_note.is_valid)

    def test_should_is_valid_false_when_currency_is_invalid(self):
        number = 1
        due_date = date.today()
        value = 100.99
        currency = ''
        city_payment = 'New York'
        state_payment = 'New York'
        issuance_date = date.today()

        beneficiary = Beneficiary(name=Name('Steve'),
                                  cpf=Cpf('11111111199'),
                                  email=Email('captain@marvel.com'))

        emitter = Emitter(name=Name('Tony Stark'),
                          cpf=Cpf('11111111188'),
                          address='New York',
                          email=Email('iron_man@marvel.com.br'))

        promissory_note = PromissoryNote(number=number,
                                         due_date=due_date,
                                         value=value,
                                         currency=currency,
                                         city_payment=city_payment,
                                         state_payment=state_payment,
                                         issuance_date=issuance_date,
                                         beneficiary=beneficiary,
                                         emitter=emitter)

        self.assertFalse(promissory_note.is_valid)

    def test_should_is_valid_false_when_city_payment_is_invalid(self):
        number = 1
        due_date = date.today()
        value = 100.99
        currency = 'real'
        city_payment = ''
        state_payment = 'New York'
        issuance_date = date.today()

        beneficiary = Beneficiary(name=Name('Steve'),
                                  cpf=Cpf('11111111199'),
                                  email=Email('captain@marvel.com'))

        emitter = Emitter(name=Name('Tony Stark'),
                          cpf=Cpf('11111111188'),
                          address='New York',
                          email=Email('iron_man@marvel.com.br'))

        promissory_note = PromissoryNote(number=number,
                                         due_date=due_date,
                                         value=value,
                                         currency=currency,
                                         city_payment=city_payment,
                                         state_payment=state_payment,
                                         issuance_date=issuance_date,
                                         beneficiary=beneficiary,
                                         emitter=emitter)

        self.assertFalse(promissory_note.is_valid)

    def test_should_is_valid_false_when_state_payment_is_invalid(self):
        number = 1
        due_date = date.today()
        value = 100.99
        currency = 'real'
        city_payment = 'New York'
        state_payment = ''
        issuance_date = date.today()

        beneficiary = Beneficiary(name=Name('Steve'),
                                  cpf=Cpf('11111111199'),
                                  email=Email('captain@marvel.com'))

        emitter = Emitter(name=Name('Tony Stark'),
                          cpf=Cpf('11111111188'),
                          address='New York',
                          email=Email('iron_man@marvel.com.br'))

        promissory_note = PromissoryNote(number=number,
                                         due_date=due_date,
                                         value=value,
                                         currency=currency,
                                         city_payment=city_payment,
                                         state_payment=state_payment,
                                         issuance_date=issuance_date,
                                         beneficiary=beneficiary,
                                         emitter=emitter)

        self.assertFalse(promissory_note.is_valid)

    def test_should_is_valid_false_when_issuance_date_is_invalid(self):
        number = 1
        due_date = date.today()
        value = 100.99
        currency = 'real'
        city_payment = 'New York'
        state_payment = 'New York'
        issuance_date = date.today() - timedelta(days=1)

        beneficiary = Beneficiary(name=Name('Steve'),
                                  cpf=Cpf('11111111199'),
                                  email=Email('captain@marvel.com'))

        emitter = Emitter(name=Name('Tony Stark'),
                          cpf=Cpf('11111111188'),
                          address='New York',
                          email=Email('iron_man@marvel.com.br'))

        promissory_note = PromissoryNote(number=number,
                                         due_date=due_date,
                                         value=value,
                                         currency=currency,
                                         city_payment=city_payment,
                                         state_payment=state_payment,
                                         issuance_date=issuance_date,
                                         beneficiary=beneficiary,
                                         emitter=emitter)

        self.assertFalse(promissory_note.is_valid)

    def test_should_add_subscriber_when_add_subscribers_is_called_with_a_callable(self):
        email_service = Mock()

        self._promissory_note.attach(email_service)

        self.assertEqual(1, len(self._promissory_note.subscribers))

    def test_should_call_subscribers_with_promissory_note_issued_when_issue_is_called_and_promissory_is_valid(self):
        promissory_note_issued = PromissoryNoteIssued(number=1,
                                                      due_date=date.today(),
                                                      value=100.99,
                                                      beneficiary_name='Steve',
                                                      beneficiary_cpf='11111111199',
                                                      beneficiary_email='captain@marvel.com',
                                                      currency='real',
                                                      city_payment='New York',
                                                      state_payment='New York',
                                                      emitter_name='Tony Stark',
                                                      emitter_cpf='11111111188',
                                                      emitter_address='New York',
                                                      emitter_email='iron_man@marvel.com',
                                                      issuance_date=date.today())

        email_service = Mock()

        self._promissory_note.attach(email_service)

        self._promissory_note.issue()

        email_service.assert_called_once_with(promissory_note_issued)

    def test_should_call_subscribers_with_promissory_note_not_issued_when_issue_is_called_and_promissory_is_valid(self):
        number = 0
        due_date = date.today()
        value = 100.99
        currency = 'real'
        city_payment = 'New York'
        state_payment = 'New York'
        issuance_date = date.today()

        beneficiary = Beneficiary(name=Name('Steve'),
                                  cpf=Cpf('11111111199'),
                                  email=Email('captain@marvel.com'))

        emitter = Emitter(name=Name('Tony Stark'),
                          cpf=Cpf('11111111188'),
                          address='New York',
                          email=Email('iron_man@marvel.com.br'))

        promissory_note = PromissoryNote(number=number,
                                         due_date=due_date,
                                         value=value,
                                         currency=currency,
                                         city_payment=city_payment,
                                         state_payment=state_payment,
                                         issuance_date=issuance_date,
                                         beneficiary=beneficiary,
                                         emitter=emitter)

        promissory_note_not_issued = PromissoryNoteNotIssued(notifications=promissory_note.notifications)

        email_service = Mock()

        promissory_note.attach(email_service)

        promissory_note.issue()

        email_service.assert_called_once_with(promissory_note_not_issued)



from dataclasses import dataclass
from datetime import date


@dataclass
class IssuePromissoryNoteCommand:
    number: int
    due_date: date
    value: float
    beneficiary_name: str
    beneficiary_cpf: str
    beneficiary_email: str
    currency: str
    city_payment: str
    state_payment: str
    emitter_name: str
    emitter_cpf: str
    emitter_address: str
    emitter_email: str
    issuance_date: date



from pyflunt.notifications import Notifiable
from pyflunt.validations import Contract


class Name(Notifiable):
    def __init__(self, name):
        super().__init__()

        self._name = name

        contract = Contract().requires().is_not_none_or_white_space(value=name,
                                                                    field='name',
                                                                    message='invalid name')
        self.add_notifications(contract)

    def __str__(self):
        return self._name


class Cpf(Notifiable):
    def __init__(self, cpf):
        super().__init__()

        self._cpf = cpf

        contract = Contract().requires().has_len(value=cpf,
                                                 length=11,
                                                 field='cpf',
                                                 message='invalid cpf')

        self.add_notifications(contract)

    def __str__(self):
        return self._cpf


class Email(Notifiable):
    def __init__(self, email):
        super().__init__()

        self._email = email

        contract = Contract().requires().is_email(value=email,
                                                  field='email',
                                                  message='invalid email')

        self.add_notifications(contract)

    def __str__(self):
        return self._email



import locale
from os import path
from os.path import dirname

from PIL import Image, ImageDraw, ImageFont

from promissory_note.events import PromissoryNoteIssued

locale.setlocale(locale.LC_ALL, 'pt_BR.UTF-8')

ROOT_DIR = dirname(dirname(__file__))

CONTENT_DIR = path.join(ROOT_DIR, 'content')

PROMISSORY_NOTE_IMAGE = path.join(CONTENT_DIR, 'promissory_note.jpg')

OPEN_SANS = path.join(CONTENT_DIR, 'open_sans.ttf')

TEXT_FONT = ImageFont.truetype(OPEN_SANS, size=16)

TEXT_COLOR = 'rgb(0,0,0)'

NUMBER_POSITION = 200, 34

DUE_DATE_DAY_POSITION = 348, 34

DUE_DATE_MONTH_POSITION = 410, 34

DUE_DATE_YEAR_POSITION = 550, 34

LONG_DUE_DATE_POSITION = 205, 62

VALUE_POSITION = 650, 34

FULL_VALUE_POSITION = 370, 148

BENEFICIARY_NAME_POSITION = 180, 118

BENEFICIARY_NAME_CPF_POSITION = 615, 118

CITY_AND_STATE_PAYMENT_POSITION = 425, 210

EMITTER_NAME_POSITION = 225, 238

ISSUANCE_DATE_DAY_POSITION = 650, 238

ISSUANCE_DATE_MONTH_POSITION = 695, 238

ISSUANCE_DATE_YEAR_POSITION = 735, 238

EMITTER_CPF_POSITION = 225, 264

EMITTER_ADDRESS_POSITION = 525, 264


def write(fn):
    def wrapper(self, *args, **kwargs):
        fn(self, *args, **kwargs)
        self._write()

    return wrapper


class PillowImageGenerationService:
    def __init__(self):
        self._image = None
        self._draw = None
        self._position = None
        self._text = None
        self._filename = None
        self._promissory_note_issued = None

    def __call__(self, event):
        if not isinstance(event, PromissoryNoteIssued):
            return

        self.generate(event)

    def generate(self, promissory_note_issued):
        self._promissory_note_issued = promissory_note_issued

        self._open_image()

        self._write_number()
        self._write_due_date_day()
        self._write_due_date_month()
        self._write_due_date_year()
        self._write_long_due_date()
        self._write_value()
        self._write_full_value()
        self._write_beneficiary_name()
        self._write_beneficiary_cpf()
        self._write_city_and_state_payment()
        self._write_emitter_name()
        self._write_issuance_date_day()
        self._write_issuance_date_month()
        self._write_issuance_date_year()
        self._write_emitter_cpf()
        self._write_emitter_address()

        self._create_filename()

        self._save_image()

    def _open_image(self):
        self._image = Image.open(PROMISSORY_NOTE_IMAGE)
        self._draw = ImageDraw.Draw(self._image)

    @write
    def _write_number(self):
        self._position = NUMBER_POSITION
        self._text = str(self._promissory_note_issued.number)

    @write
    def _write_due_date_day(self):
        self._position = DUE_DATE_DAY_POSITION
        self._text = self._promissory_note_issued.due_date.strftime('%d')

    @write
    def _write_due_date_month(self):
        self._position = DUE_DATE_MONTH_POSITION
        self._text = self._promissory_note_issued.due_date.strftime('%B')

    @write
    def _write_due_date_year(self):
        self._position = DUE_DATE_YEAR_POSITION
        self._text = self._promissory_note_issued.due_date.strftime('%Y')

    @write
    def _write_long_due_date(self):
        self._position = LONG_DUE_DATE_POSITION
        self._text = self._promissory_note_issued.due_date.strftime('%d de %B de %Y')

    @write
    def _write_value(self):
        self._position = VALUE_POSITION
        self._text = f'{self._promissory_note_issued.value:.2f}'

    @write
    def _write_full_value(self):
        self._position = FULL_VALUE_POSITION
        self._text = f'R$ {self._promissory_note_issued.value:.2f}'

    @write
    def _write_beneficiary_name(self):
        self._position = BENEFICIARY_NAME_POSITION
        self._text = self._promissory_note_issued.beneficiary_name

    @write
    def _write_beneficiary_cpf(self):
        self._position = BENEFICIARY_NAME_CPF_POSITION
        self._text = self._promissory_note_issued.beneficiary_cpf

    @write
    def _write_city_and_state_payment(self):
        self._position = CITY_AND_STATE_PAYMENT_POSITION
        self._text = f'{self._promissory_note_issued.city_payment} - {self._promissory_note_issued.state_payment}'

    @write
    def _write_emitter_name(self):
        self._position = EMITTER_NAME_POSITION
        self._text = self._promissory_note_issued.emitter_name

    @write
    def _write_issuance_date_day(self):
        self._position = ISSUANCE_DATE_DAY_POSITION
        self._text = self._promissory_note_issued.issuance_date.strftime('%d')

    @write
    def _write_issuance_date_month(self):
        self._position = ISSUANCE_DATE_MONTH_POSITION
        self._text = self._promissory_note_issued.issuance_date.strftime('%m')

    @write
    def _write_issuance_date_year(self):
        self._position = ISSUANCE_DATE_YEAR_POSITION
        self._text = self._promissory_note_issued.issuance_date.strftime('%y')

    @write
    def _write_emitter_cpf(self):
        self._position = EMITTER_CPF_POSITION
        self._text = self._promissory_note_issued.emitter_cpf

    @write
    def _write_emitter_address(self):
        self._position = EMITTER_ADDRESS_POSITION
        self._text = self._promissory_note_issued.emitter_address

    def _create_filename(self):
        basename = f'{self._promissory_note_issued.number}_promissory_note.jpg'
        self._filename = path.join(CONTENT_DIR, basename)

    def _save_image(self):
        self._image.save(self._filename)

    def _write(self):
        self._draw.text(self._position, self._text, fill=TEXT_COLOR, font=TEXT_FONT)



from promissory_note.entities import Beneficiary, Emitter, PromissoryNote
from promissory_note.value_objects import Name, Cpf, Email


class IssuePromissoryNote:
    def __init__(self, image_generation_service, email_service):
        self.image_generation_service = image_generation_service
        self._email_service = email_service

    def execute(self, command):
        beneficiary = Beneficiary(name=Name(command.beneficiary_name),
                                  cpf=Cpf(command.beneficiary_cpf),
                                  email=Email(command.beneficiary_email))

        emitter = Emitter(name=Name(command.emitter_name),
                          cpf=Cpf(command.emitter_cpf),
                          address=command.emitter_address,
                          email=Email(command.emitter_email))

        promissory_note = PromissoryNote(number=command.number,
                                         due_date=command.due_date,
                                         value=command.value,
                                         currency=command.currency,
                                         city_payment=command.city_payment,
                                         state_payment=command.state_payment,
                                         issuance_date=command.issuance_date,
                                         beneficiary=beneficiary,
                                         emitter=emitter)

        promissory_note.attach(self.image_generation_service, self._email_service)

        promissory_note.issue()



from datetime import date

from pyflunt.notifications import Notifiable
from pyflunt.validations import Contract

from promissory_note.events import PromissoryNoteIssued, PromissoryNoteNotIssued


class Beneficiary(Notifiable):
    def __init__(self, name, cpf, email):
        super().__init__()

        self._name = name
        self._cpf = cpf
        self._email = email

        self.add_notifications(name, cpf, email)

    @property
    def name(self):
        return str(self._name)

    @property
    def cpf(self):
        return str(self._cpf)

    @property
    def email(self):
        return str(self._email)


class Emitter(Notifiable):
    def __init__(self, name, cpf, address, email):
        super().__init__()

        self._name = name
        self._cpf = cpf
        self._address = address
        self._email = email

        contract = Contract().requires().is_not_none_or_white_space(value=address,
                                                                    field='address',
                                                                    message='invalid address')

        self.add_notifications(contract, name, cpf, email)

    @property
    def name(self):
        return str(self._name)

    @property
    def cpf(self):
        return str(self._cpf)

    @property
    def address(self):
        return self._address

    @property
    def email(self):
        return str(self._email)


class PromissoryNote(Notifiable):
    def __init__(self,
                 number,
                 due_date,
                 value,
                 currency,
                 city_payment,
                 state_payment,
                 issuance_date,
                 beneficiary,
                 emitter):
        super().__init__()

        self._number = number
        self._due_date = due_date
        self._value = value
        self._currency = currency
        self._city_payment = city_payment
        self._state_payment = state_payment
        self._issuance_date = issuance_date
        self._beneficiary = beneficiary
        self._emitter = emitter
        self._subscribers = []
        self._event = None

        contract = (Contract().requires()
                    .is_greater_than(value=number,
                                     comparer=0,
                                     field='number',
                                     message='number should be greater than zero')
                    .is_greater_or_equals_than(value=due_date,
                                               comparer=date.today(),
                                               field='due date',
                                               message='date should be greater or equals than today')
                    .is_greater_than(value=value,
                                     comparer=0,
                                     field='value',
                                     message='value should be greater than zero')
                    .is_not_none_or_white_space(value=currency,
                                                field='currency',
                                                message='invalid currency')
                    .is_not_none_or_white_space(value=city_payment,
                                                field='city payment',
                                                message='invalid city payment')
                    .is_not_none_or_white_space(value=state_payment,
                                                field='state payment',
                                                message='invalid state payment')
                    .is_greater_or_equals_than(value=issuance_date,
                                               comparer=date.today(),
                                               field='issuance date',
                                               message='issuance date should be greater or equals than today'))

        self.add_notifications(contract, beneficiary, emitter)

    @property
    def subscribers(self):
        return tuple(self._subscribers)

    def issue(self):
        self._create_event()
        self._notify()

    def attach(self, *subscribers):
        for subscriber in subscribers:
            self._subscribers.append(subscriber)

    def _create_event(self):
        if not self.is_valid:
            self._event = PromissoryNoteNotIssued(notifications=self.notifications)
            return

        self._event = PromissoryNoteIssued(number=self._number,
                                           due_date=self._due_date,
                                           value=self._value,
                                           beneficiary_name=self._beneficiary.name,
                                           beneficiary_cpf=self._beneficiary.cpf,
                                           beneficiary_email=self._beneficiary.email,
                                           currency=self._currency,
                                           city_payment=self._city_payment,
                                           state_payment=self._state_payment,
                                           emitter_name=self._emitter.name,
                                           emitter_cpf=self._emitter.cpf,
                                           emitter_address=self._emitter.address,
                                           emitter_email=self._emitter.email,
                                           issuance_date=self._issuance_date)

    def _notify(self):
        for it in self._subscribers:
            it(self._event)
